package repositories

import (
	"context"

	"git.dmitriygnatenko.ru/dima/go-common/db"
	sq "github.com/Masterminds/squirrel"
)

const articleTagTableName = "article_tag"

type ArticleTagRepository struct {
	db *db.DB
}

func InitArticleTagRepository(db *db.DB) *ArticleTagRepository {
	return &ArticleTagRepository{db: db}
}

func (a ArticleTagRepository) Add(ctx context.Context, articleID uint64, tagIDs []uint64) error {
	if len(tagIDs) == 0 {
		return nil
	}

	qb := sq.Insert(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Columns("article_id", "tag_id")

	for _, tagID := range tagIDs {
		qb = qb.Values(articleID, tagID)
	}

	query, args, err := qb.ToSql()
	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}

func (a ArticleTagRepository) Delete(ctx context.Context, articleID uint64, tagIDs []uint64) error {
	if len(tagIDs) == 0 {
		return nil
	}

	query, args, err := sq.Delete(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"article_id": articleID}).
		Where(sq.Eq{"tag_id": tagIDs}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}

func (a ArticleTagRepository) DeleteByArticleID(ctx context.Context, articleID uint64) error {
	query, args, err := sq.Delete(articleTagTableName).
		PlaceholderFormat(sq.Dollar).
		Where(sq.Eq{"article_id": articleID}).
		ToSql()

	if err != nil {
		return err
	}

	_, err = a.db.ExecContext(ctx, query, args...)

	return err
}
