package handler

import (
	"database/sql"
	"errors"

	"git.dmitriygnatenko.ru/dima/go-common/logger"
	"github.com/gofiber/fiber/v2"

	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
)

const (
	TagCacheKey = "tag"
	tagParam    = "tag"
)

func TagHandler(
	cacheService CacheService,
	articleRepository ArticleRepository,
	tagRepository TagRepository,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		url := fctx.Params(tagParam)

		renderData, found := cacheService.Get(TagCacheKey + url)

		if !found {
			tag, err := tagRepository.GetByURL(ctx, url)
			if err != nil {
				if errors.Is(err, sql.ErrNoRows) {
					return fiber.ErrNotFound
				}

				logger.Error(ctx, err.Error())

				return err
			}

			articles, err := articleRepository.GetPreviewByTagID(ctx, tag.ID)
			if err != nil {
				logger.Error(ctx, err.Error())
				return err
			}

			renderData = fiber.Map{
				"headTitle":       "От слона к суслику - статьи про PHP, Go, алгоритмы",
				"headDescription": "статьи с тегом " + tag.Tag,
				"headKeywords":    "программирование, статьи, блог, " + tag.Tag,
				"pageTitle":       "Тег<br>" + tag.Tag,
				"articles":        mapper.ToArticlePreviewDTOList(articles),
			}

			cacheService.Set(TagCacheKey+url, renderData, nil)
		}

		return fctx.Render("tag", renderData, "_layout")
	}
}
